@@ -6,12 +6,16 @@
     :copyright: (c) 2013 by Heungsub Lee
     :license: BSD, see LICENSE for more details.
 """
+from contextlib import contextmanager
+
+import zmq
 
 
-__all__ = ['ZeronimoException', 'WorkerNotFound', 'make_worker_not_found']
+__all__ = ['ZeronimoException', 'WorkerNotFound', 'UnexpectedMessage',
+           'make_worker_not_found', 'raises']
 
 
-class ZeronimoException(Exception):
+class ZeronimoException(BaseException):
     """The base exception for exceptions Zeronimo customized."""
 
     pass
@@ -25,6 +29,18 @@ class WorkerNotFound(ZeronimoException, LookupError):
     pass
 
 
+class SocketClosed(ZeronimoException, zmq.ZMQError):
+    """Collector socket is closed while a task is collecting replies."""
+
+    pass
+
+
+class UnexpectedMessage(ZeronimoException, RuntimeWarning):
+    """Warns when a received message is not expected format."""
+
+    pass
+
+
 def make_worker_not_found(rejected=0):
     """Generates an error message by the count of workers rejected for
     :exc:`WorkerNotFound`.
@@ -41,6 +57,20 @@ def make_worker_not_found(rejected=0):
         errmsg.append('a worker rejected')
     elif rejected:
         errmsg.append('{0} workers rejected'.format(rejected))
-    err = WorkerNotFound(', '.join(errmsg))
-    err.rejected = rejected
-    return err
+    exc = WorkerNotFound(', '.join(errmsg))
+    exc.rejected = rejected
+    return exc
+
+
+@contextmanager
+def raises(exctype):
+    """Expected exceptions occured in the context will not raised at the
+    worker's greenlet. The exceptions will still be delivered to the collector.
+
+    Use this in methods of the application object.
+    """
+    try:
+        yield
+    except exctype as exc:
+        exc._znm_expected = True
+        raise
