@@ -1,18 +1,24 @@
 # -*- coding: utf-8 -*-
 import time
+import warnings
+import weakref
 
 import gevent
 from gevent import joinall, spawn
 import pytest
 import zmq.green as zmq
 
-from conftest import Application, link_sockets, run_device, sync_pubsub
+from conftest import (
+    Application, link_sockets, run_device, running, sync_pubsub)
 import zeronimo
 
 
+warnings.simplefilter('always')
+
+
 def test_running():
-    from zeronimo.components import Runnable
-    class NullRunner(Runnable):
+    from zeronimo.components import Component
+    class NullRunner(Component):
         def run(self):
             gevent.sleep(0.1)
     runner = NullRunner()
@@ -43,40 +49,36 @@ def test_messaging(ctx, addr, topic):
 
 
 def test_from_socket(ctx, addr1, addr2):
-    try:
-        # sockets
-        worker_sock = ctx.socket(zmq.PULL)
-        worker_sock.bind(addr1)
-        collector_sock = ctx.socket(zmq.PULL)
-        collector_sock.bind(addr2)
-        push = ctx.socket(zmq.PUSH)
-        push.connect(addr1)
-        # components
-        app = Application()
-        worker = zeronimo.Worker(app, [worker_sock])
-        worker.start()
-        collector = zeronimo.Collector(collector_sock, addr2)
-        customer = zeronimo.Customer(push, collector)
+    # sockets
+    worker_sock = ctx.socket(zmq.PULL)
+    worker_sock.bind(addr1)
+    collector_sock = ctx.socket(zmq.PULL)
+    collector_sock.bind(addr2)
+    push = ctx.socket(zmq.PUSH)
+    push.connect(addr1)
+    # components
+    app = Application()
+    worker = zeronimo.Worker(app, [worker_sock])
+    collector = zeronimo.Collector(collector_sock, addr2)
+    customer = zeronimo.Customer(push, collector)
+    with running([worker], sockets=[worker_sock, collector_sock, push]):
         # test
         assert customer.zeronimo() == 'zeronimo'
-    finally:
-        try:
-            collector.stop()
-            collector_sock.close()
-            push.close()
-            worker.stop()
-            worker_sock.close()
-        except UnboundLocalError:
-            pass
 
 
 def test_socket_type_error(ctx):
     with pytest.raises(ValueError):
-        zeronimo.Customer(ctx.socket(zmq.PAIR))
+        zeronimo.Customer(ctx.socket(zmq.REQ))
     with pytest.raises(ValueError):
-        zeronimo.Worker(None, [ctx.socket(zmq.PAIR)])
+        zeronimo.Customer(ctx.socket(zmq.REP))
     with pytest.raises(ValueError):
-        zeronimo.Collector(ctx.socket(zmq.PAIR), 'x')
+        zeronimo.Worker(None, [ctx.socket(zmq.REQ)])
+    with pytest.raises(ValueError):
+        zeronimo.Worker(None, [ctx.socket(zmq.REP)])
+    with pytest.raises(ValueError):
+        zeronimo.Collector(ctx.socket(zmq.REQ), 'x')
+    with pytest.raises(ValueError):
+        zeronimo.Collector(ctx.socket(zmq.REP), 'x')
 
 
 def test_fixtures(worker, push, pub, collector, addr1, addr2, ctx):
@@ -307,12 +309,12 @@ def test_device(ctx, collector, topic, addr1, addr2, addr3, addr4):
 def test_x_forwarder(ctx, collector, topic, addr1, addr2):
     # customer  |----| forwarder with XPUB/XSUB |---> | worker
     # collector | <-----------------------------------|
+    # run forwarder
+    forwarder_in_addr, forwarder_out_addr = addr1, addr2
+    forwarder = spawn(
+        run_device, ctx.socket(zmq.XSUB), ctx.socket(zmq.XPUB),
+        forwarder_in_addr, forwarder_out_addr)
     try:
-        # run forwarder
-        forwarder_in_addr, forwarder_out_addr = addr1, addr2
-        forwarder = spawn(
-            run_device, ctx.socket(zmq.XSUB), ctx.socket(zmq.XPUB),
-            forwarder_in_addr, forwarder_out_addr)
         forwarder.join(0)
         # connect to the devices
         worker_sub1 = ctx.socket(zmq.SUB)
@@ -328,19 +330,12 @@ def test_x_forwarder(ctx, collector, topic, addr1, addr2):
         app = Application()
         worker1 = zeronimo.Worker(app, [worker_sub1])
         worker2 = zeronimo.Worker(app, [worker_sub2])
-        worker1.start()
-        worker2.start()
-        # zeronimo!
-        fanout_customer = zeronimo.Customer(pub, collector)[topic]
-        assert fanout_customer.zeronimo() == ['zeronimo', 'zeronimo']
+        with running([worker1, worker2], sockets=[pub]):
+            # zeronimo!
+            fanout_customer = zeronimo.Customer(pub, collector)[topic]
+            assert fanout_customer.zeronimo() == ['zeronimo', 'zeronimo']
     finally:
-        try:
-            forwarder.kill()
-            pub.close()
-            worker1.stop()
-            worker2.stop()
-        except UnboundLocalError:
-            pass
+        forwarder.kill()
 
 
 def test_proxied_collector(ctx, worker, push, addr1, addr2):
@@ -413,3 +408,97 @@ def test_unreachable(worker, collector, push, pub, topic):
         with pytest.raises(zeronimo.WorkerNotFound):
             fanout_customer.zeronimo()
         assert customer_nowait.zeronimo() is None
+
+
+def test_expected_exception(capsys, worker, collector, push):
+    customer = zeronimo.Customer(push, collector)
+    with pytest.raises(ZeroDivisionError):
+        customer.zero_div()
+    out, err = capsys.readouterr()
+    assert 'Traceback (most recent call last)' in err
+    assert 'ZeroDivisionError' in err
+    with pytest.raises(ZeroDivisionError):
+        customer.ignore_exc(ZeroDivisionError, ValueError)
+    out, err = capsys.readouterr()
+    assert 'Traceback (most recent call last)' in err
+    assert 'ZeroDivisionError' in err
+    with pytest.raises(ZeroDivisionError):
+        customer.ignore_exc(ZeroDivisionError, ZeroDivisionError)
+    out, err = capsys.readouterr()
+    assert not err
+    with pytest.raises(ZeroDivisionError):
+        customer.ignore_exc(ZeroDivisionError, (ArithmeticError, ValueError))
+    out, err = capsys.readouterr()
+    assert not err
+
+
+def test_pgm_connect(ctx, fanout_addr):
+    if 'pgm' not in fanout_addr[:4]:
+        pytest.skip()
+    sub = ctx.socket(zmq.SUB)
+    sub.set(zmq.SUBSCRIBE, '')
+    sub.connect(fanout_addr)
+    worker = zeronimo.Worker(None, [sub])
+    worker.start()
+    pub1 = ctx.socket(zmq.PUB)
+    pub1.connect(fanout_addr)
+    pub2 = ctx.socket(zmq.PUB)
+    pub2.connect(fanout_addr)  # in zmq-3.2, pgm-connect sends an empty message
+    try:
+        worker.wait(0.1)
+    finally:
+        worker.stop()
+        sub.close()
+        pub1.close()
+        pub2.close()
+
+
+def test_unexpected_message(worker, push):
+    push.send('Zeronimo!')  # indicates to ExtraData
+    push.send('')  # indicates to UnpackValueError
+    with warnings.catch_warnings(record=True) as w:
+        gevent.sleep(0.1)
+    assert len(w) == 2
+    assert w[0].category is zeronimo.UnexpectedMessage
+    assert w[1].category is zeronimo.UnexpectedMessage
+    assert w[0].message.serial == 'Zeronimo!'
+    assert w[1].message.serial == ''
+
+
+def test_queue_leaking(worker, task_collector, push):
+    customer = zeronimo.Customer(push, task_collector)
+    task = customer.zeronimo()
+    reply_queue_ref = weakref.ref(task.reply_queue)
+    assert task_collector.reply_queues
+    assert task_collector.missing_queues
+    assert task() == 'zeronimo'
+    assert not task_collector.reply_queues
+    assert not task_collector.missing_queues
+    del task
+    assert reply_queue_ref() is None
+
+
+def test_task_broken(worker, task_collector, push):
+    customer = zeronimo.Customer(push, task_collector)
+    task = customer.sleep(0.1)
+    task_collector.socket.close()
+    with pytest.raises(zeronimo.SocketClosed):
+        task()
+
+
+def test_pair(ctx, addr):
+    left = ctx.socket(zmq.PAIR)
+    right = ctx.socket(zmq.PAIR)
+    left.bind(addr)
+    right.connect(addr)
+    worker = zeronimo.Worker(Application(), [left])
+    with pytest.raises(ValueError):  # pair collector doesn't need an address
+        zeronimo.Collector(right, addr)
+    collector = zeronimo.Collector(right)
+    customer = zeronimo.Customer(right, collector)
+    with running([worker], sockets=[left, right]):
+        assert customer.zeronimo() == 'zeronimo'
+        assert ' '.join(customer.rycbar123()) == \
+               'run, you clever boy; and remember.'
+        with pytest.raises(ZeroDivisionError):
+            customer.zero_div()
